//Importing from d3
import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  format,
  line,
  area,
  curveBasis
} from 'd3';

const svg = select('svg');

//Fixing width and height
const width = +svg.attr('width');
const height = +svg.attr('height');

// Formatting the total value to show on the Areachart
const xAxisTickFormat = number =>
    format('.3s')(number)

const render = data => {
  
  const xValue = d => d.Year;
  const xAxisLabel = 'Year';
  
  const yValue = d => d.Annual_NetIncome_In_Millions;
  const yAxisLabel = 'Annual_NetIncome_In_Millions';
  
  const margin = { top: 10, right: 40, bottom: 88, left: 120 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top-30 - margin.bottom-20; 
  
// Determing the Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
// Determing the Scale for Y axis  
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  
 // Positioning the svg element   
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
 // Assigning values to x axis 
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
 // Assigning values to y axis  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(xAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -90)
      .attr('x', -innerHeight / 2)
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
 
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
 xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('x', innerWidth / 2)
      .text(xAxisLabel); 
  
 // To draw the line on the curve   and adding it to svg 
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
 // To draw the area under the curve and adding it to svg 
  const areaGenerator = area()
  	.x(d => xScale(xValue(d)))
  	.y0(innerHeight)
  	.y1(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
  g.append('path')
  	.attr('class', 'line-chart')
  	.attr('d', lineGenerator(data));
  
  g.append('path')
  	.attr('class', 'area-chart')
  	.attr('d', areaGenerator(data));
  
 
};

// Loading and Parsing CSV data
csv('https://gist.githubusercontent.com/nagasai-gummadi/6d6c370a943222612293338107d2e7c3/raw/24e70ff59bb4dd7c198d17b0dd42d6ee4c2f3744/income.csv')
  .then(data => {
    data.forEach(d => {
      d.Annual_NetIncome_In_Millions = +d.Annual_NetIncome_In_Millions;
      
//Formatting to retrieve year from the date
      d.Year = d3.timeParse("%Y-%m-%d")(d.Year);
    });
    render(data);
  });